unit synaoauth;

interface
uses
  classes, synacode, synautil, sysutils, DateUtils;

type
  TSynaOauth = class(TObject)
  protected
    par: Tstringlist;
    specchar: TSpecials;
    function GenerateNonce: string;
    function GenerateTimeStamp: string;
    function EncodePar(value: string): string;
  public
    askmethod: string;
    consumer_key: string;
    consumer_secret: string;
    signature_method: string;
    callback: string;
    request_token: string;
    request_token_secret: string;
    verifier: string;
    access_token: string;
    access_token_secret: string;

    basestring: string;

    constructor Create;
    destructor Destroy; override;
    procedure clear;

    function BuildRequest(key, secret, url, callbackurl: string): string;
    function ParseRequest(value: string): boolean;
    function ParseVerifier(value: string): boolean;
    function BuildAccess(url: string): string;
    function ParseAccess(value: string): string;
  end;

implementation

function SortNames(List: TStringList; Index1, Index2: Integer): Integer;
var
  s1, s2: string;
begin
  s1 := List.Names[index1];
  s2 := List.Names[index2];
  Result := AnsiCompareStr(s1, s2);
end;

{ TSynaOauth }

function TSynaOauth.BuildAccess(url: string): string;
var
  s: string;
  sig: string;
  n: integer;
begin
  par.clear;
  par.add('oauth_consumer_key=' + EncodePar(Consumer_Key));
  par.add('oauth_token=' + EncodePar(request_token));
  par.Add('oauth_nonce=' + EncodePar(Self.GenerateNonce));
  par.Add('oauth_timestamp=' + EncodePar(Self.GenerateTimeStamp));
  par.Add('oauth_version=' + EncodePar('1.0'));
  par.add('oauth_verifier=' + EncodePar(verifier));
  if signature_method <> 'HMAC-SHA1' then
    signature_method := 'PLAINTEXT';
  par.Add('oauth_signature_method=' + EncodePar(signature_method));
  par.CustomSort(SortNames);
  basestring := askmethod + '&' + EncodePar(url) + '&';
  s := '';
  for n := 0 to par.count - 1 do
  begin
    if s <> '' then
      s := s + '&';
    s := s + par[n];
  end;
  s := EncodePar(s);
  basestring := basestring + s;
  s := EncodePar(consumer_secret) + '&' + EncodePar(request_token_secret);
  sig := '';
  if signature_method = 'PLAINTEXT' then
    sig := s;
  if signature_method = 'HMAC-SHA1' then
    sig := EncodeBase64(HMAC_SHA1(BaseString, s));

  result := '';
  for n := 0 to par.count - 1 do
  begin
    if result <> '' then
      result := result + '&';
    result := result + par[n];
  end;
  result := 'oauth_signature=' + EncodePar(sig) + '&' + result;
end;

function TSynaOauth.BuildRequest(key, secret, url, callbackurl: string): string;
var
  s: string;
  sig: string;
  n: integer;
begin
  consumer_key := key;
  consumer_secret := secret;
  callback := callbackurl;
  par.clear;
  if callback <> '' then
    par.add('oauth_callback=' + EncodePar(callback));
  par.add('oauth_consumer_key=' + EncodePar(Consumer_Key));
  par.Add('oauth_nonce=' + EncodePar(Self.GenerateNonce));
  par.Add('oauth_timestamp=' + EncodePar(Self.GenerateTimeStamp));
  par.Add('oauth_version=' + EncodePar('1.0'));
  if signature_method <> 'HMAC-SHA1' then
    signature_method := 'PLAINTEXT';
  par.Add('oauth_signature_method=' + EncodePar(signature_method));
  par.CustomSort(SortNames);
  basestring := askmethod + '&' + EncodePar(url) + '&';
  s := '';
  for n := 0 to par.count - 1 do
  begin
    if s <> '' then
      s := s + '&';
    s := s + par[n];
  end;
  s := EncodePar(s);
  basestring := basestring + s;
  s := EncodePar(consumer_secret) + '&';
  sig := '';
  if signature_method = 'PLAINTEXT' then
    sig := s;
  if signature_method = 'HMAC-SHA1' then
    sig := EncodeBase64(HMAC_SHA1(BaseString, s));

  result := '';
  for n := 0 to par.count - 1 do
  begin
    if result <> '' then
      result := result + '&';
    result := result + par[n];
  end;
  result := 'oauth_signature=' + EncodePar(sig) + '&' + result;
end;

procedure TSynaOauth.clear;
begin
  askmethod := 'POST';
  consumer_key := '';
  consumer_secret := '';
  signature_method := 'HMAC-SHA1';
  callback := '';
  request_token := '';
  request_token_secret := '';
  verifier := '';
  access_token := '';
  access_token_secret := '';
  par.clear;
end;

constructor TSynaOauth.Create;
begin
  inherited;
  par := Tstringlist.create;
  clear;
  specchar := [#32..#255];
  specchar := specchar - ['a'..'z', 'A'..'Z', '0'..'9', '-', '.', '_', '~'];
end;

destructor TSynaOauth.Destroy;
begin
  par.free;
  inherited;
end;

function TSynaOauth.EncodePar(value: string): string;
begin
  Result := EncodeTriplet(value, '%', specchar);
end;

function TSynaOauth.GenerateNonce: string;
begin
  Result := inttohex(gettick, 8);
end;

function TSynaOauth.GenerateTimeStamp: string;
begin
  Result := IntToStr(DateTimeToUnix(GetUTTime));
end;

function TSynaOauth.ParseAccess(value: string): string;
begin
  result := '';
  if pos('oauth_token=', value) > 0 then
  begin
    result := separateright(value, 'oauth_token=');
    result := separateleft(result, '&');
    result := decodeurl(result);
  end;
end;

function TSynaOauth.ParseRequest(value: string): boolean;
begin
  request_token := '';
  request_token_secret := '';
  if pos('oauth_token=', value) > 0 then
  begin
    request_token := separateright(value, 'oauth_token=');
    request_token := separateleft(request_token, '&');
    request_token := decodeurl(request_token);
  end;
  if pos('oauth_token_secret=', value) > 0 then
  begin
    request_token_secret := separateright(value, 'oauth_token_secret=');
    request_token_secret := separateleft(request_token_secret, '&');
    request_token_secret := decodeurl(request_token_secret);
  end;
  result := (request_token <> '') and (request_token_secret <> '');
end;

function TSynaOauth.ParseVerifier(value: string): boolean;
var
  s: string;
begin
  result := false;
  verifier := '';
  s := '';
  if pos('oauth_token=', value) > 0 then
  begin
    s := separateright(value, 'oauth_token=');
    s := separateleft(s, '&');
    s := decodeurl(s);
  end;
  if request_token <> s then
    exit;
  if pos('oauth_verifier=', value) > 0 then
  begin
    verifier := separateright(value, 'oauth_verifier=');
    verifier := separateleft(verifier, '&');
    verifier := decodeurl(verifier);
  end;
  result := verifier <> '';
end;

end.
